package solatium.blockchain.persistence;

import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import solatium.blockchain.Block;
import solatium.blockchain.HashedBlock;
import solatium.blockchain.Transaction;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class PersistenceBlockChainTest {

    private static final String TEST_RESOURCE_PATH = "./src/test/resources/persistenceTest/";
    private final File directory;

    public PersistenceBlockChainTest() {
        Path path = Paths.get(TEST_RESOURCE_PATH);
        directory = new File(path.toUri());
    }

    @Before
    public void setUp() {
        if (!directory.isDirectory()) {
            directory.mkdir();
        }
        assertTrue(directory.isDirectory());
    }

    @After
    public void cleanUp() throws IOException {
        FileUtils.cleanDirectory(directory);
    }

    @Test
    public void shouldWriteHashBlockWithHashAsName() {
        PersistenceBlockChain persistenceBlockChain = new PersistenceBlockChain(directory);

        Block block = new FakeBlock("foo");
        HashedBlock hashedBlock = block.getHashed();
        persistenceBlockChain.write(hashedBlock);

        List<File> fileList = Arrays.asList(directory.listFiles());

        assertTrue(fileList.size() > 0);
        assertEquals(1, fileList.stream().filter(file -> file.getName().equals("foo")).collect(Collectors.toList()).size());
    }

    private class FakeBlock implements Block {

        private final String hash;

        public FakeBlock(String hash) {
            this.hash = hash;
        }

        @Override
        public String getDataToHash() {
            return null;
        }

        @Override
        public HashedBlock getHashed() {
            return new HashedBlock(this, hash);
        }

        @Override
        public String getPreviousHash() {
            return null;
        }

        @Override
        public Transaction getData() {
            return null;
        }

        @Override
        public long getTimestamp() {
            return 0;
        }
    }
}
